@@ -18,10 +18,18 @@ use 5.010;
 use strict;
 use warnings;
 use base qw(Net::Statsd::Server::Backend);
+
 use AnyEvent::Log;
 use IO::Socket::INET ();
 use Time::HiRes      ();
 
+use constant {
+  fmt_FLOAT => '%.6f',
+  fmt_INT   => '%d',
+  fmt_STR   => '%s',
+  fmt_TIME  => '%d',
+};
+
 sub init {
   my ($self, $startup_time, $config) = @_;
 
@@ -38,13 +46,22 @@ sub init {
   my $prefixSet       = $config->{graphite}->{prefixSet}       // "sets";
   my $legacyNamespace = $config->{graphite}->{legacyNamespace} // 1;
 
-  my $globalNamespace  = ['stats'];
-  my $counterNamespace = ['stats'];
-  my $timerNamespace   = ['stats', 'timers'];
-  my $gaugesNamespace  = ['stats', 'gauges'];
-  my $setsNamespace    = ['stats', 'sets'];
+  my $globalNamespace  = [];
+  my $counterNamespace = [];
+  my $timerNamespace   = [];
+  my $gaugesNamespace  = [];
+  my $setsNamespace    = [];
 
-  if (! $legacyNamespace) {
+  if ($legacyNamespace) {
+
+    $globalNamespace  = ['stats'];
+    $counterNamespace = ['stats'];
+    $timerNamespace   = ['stats', 'timers'];
+    $gaugesNamespace  = ['stats', 'gauges'];
+    $setsNamespace    = ['stats', 'sets'];
+
+  }
+  else {
 
     if ($globalPrefix ne "") {
       push @{ $globalNamespace },  $globalPrefix;
@@ -91,11 +108,128 @@ sub init {
   $self->{flushInterval} = $config->{flushInterval};
 }
 
-sub post_stats {
-  my ($self, $stat_string) = @_;
+sub flush {
+  my ($self, $timestamp, $metrics) = @_;
+  my $flush_stats = $self->flush_stats($timestamp, $metrics);
+  $self->post_stats($flush_stats);
+}
+
+sub flush_stats {
+  my ($self, $ts, $metrics) = @_;
+
+  my $startTime = [ Time::HiRes::gettimeofday ];
+  my $statString = "";
+  my $num_stats = 0;
+  my $timer_data_key;
+
+  my $counters = $metrics->{counters};
+  my $gauges = $metrics->{gauges};
+  my $timers = $metrics->{timers};
+  my $sets = $metrics->{sets};
+  my $counter_rates = $metrics->{counter_rates};
+  my $timer_data = $metrics->{timer_data};
+  my $statsd_metrics = $metrics->{statsd_metrics};
+
+  # Accumulate flush statistics into a list
+  my @fstats;
+
+  for my $key (keys %{ $counters }) {
+
+    my @namespace = (@{ $self->{counterNamespace} }, $key);
+    my $namespace = join(".", @namespace);
+
+    my $value = $counters->{$key};
+    my $valuePerSecond = $counter_rates->{$key}; # pre-calculated "per second" rate
+
+    if ($self->{legacyNamespace}) {
+      push @fstats, stat_float($namespace, $valuePerSecond, $ts);
+      push @fstats, stat_int("stats_counts.$key", $value, $ts);
+    } else {
+      push @fstats, stat_float("$namespace.rate", $valuePerSecond, $ts);
+      push @fstats, stat_int("$namespace.count", $value, $ts);
+    }
+
+    $num_stats++;
+  }
+
+  for my $key (keys %{ $timer_data }) {
+    if ($timer_data->{$key} && keys %{ $timer_data->{$key} } > 0) {
+      for my $timer_data_key (keys %{ $timer_data->{$key} }) {
+        my @namespace = (@{ $self->{timerNamespace} }, $key);
+        my $the_key = join(".", @namespace);
+        push @fstats, stat_float(
+          "$the_key.$timer_data_key",
+          $timer_data->{$key}->{$timer_data_key}, $ts
+        );
+      }
+      $num_stats++;
+    }
+  }
+
+  for my $key (keys %{ $gauges }) {
+    my @namespace = (@{ $self->{gaugesNamespace} }, $key);
+    push @fstats, stat_float(join(".", @namespace), $gauges->{$key}, $ts);
+    $num_stats++;
+  }
+
+  for my $key (keys %{ $sets }) {
+    my @namespace = (@{ $self->{setsNamespace} }, $key);
+    my $set_count = join(".", @namespace, "count");
+    my $set_len = scalar keys %{ $sets->{$key} };
+    push @fstats, stat_int($set_count, $set_len, $ts);
+    $num_stats++;
+  }
+
+  my @namespace = (@{ $self->{globalNamespace} }, "statsd");
+
+  # Convert Time::HiRes format (µs) to ms
+  my $calcTime = 1000 * Time::HiRes::tv_interval($startTime);
+
+  if ($self->{legacyNamespace}) {
+    push @fstats, stat_int("statsd.numStats", $num_stats, $ts);
+    push @fstats, stat_float("stats.statsd.graphiteStats.calculationtime",
+      $calcTime, $ts);
+    for my $key (keys %{ $statsd_metrics }) {
+      push @fstats, stat_int("stats.statsd.$key", $statsd_metrics->{$key}, $ts);
+    }
+  }
+  else {
+    my $namespace = join(".", @namespace);
+    push @fstats, stat_int("${namespace}.numStats", $num_stats, $ts);
+    push @fstats, stat_float(
+      "${namespace}.graphiteStats.calculationtime", $calcTime, $ts);
+    for my $key (keys %{ $statsd_metrics }) {
+      my $value = $statsd_metrics->{$key};
+      push @fstats, stat_str("${namespace}.${key}", $value, $ts);
+    }
+  }
+
+  my $global_stats = $self->global_stats();
+  push @fstats, @{ $global_stats };
+
+  return \@fstats;
+}
+
+sub global_stats {
+  my ($self) = @_;
 
   my $last_flush = $self->{lastFlush} || 0;
   my $last_exception = $self->{lastException} || 0;
+  my $ts = time();
+
+  my @namespace = (@{ $self->{globalNamespace} }, 'statsd', 'graphiteStats');
+  my $namespace = join(".", @namespace);
+
+  my $global_stats = [
+    stat_time("${namespace}.last_exception", $last_exception, $ts),
+    stat_time("${namespace}.last_flush", $last_flush, $ts),
+  ];
+
+  return $global_stats;
+}
+
+sub post_stats {
+  my ($self, $stat_list) = @_;
 
   return if ! $self->{graphiteHost};
 
@@ -107,18 +241,7 @@ sub post_stats {
       PeerPort => $port,
     ) or die "Can't connect to Graphite on ${host}:${port}: $!";
 
-    my $ts = time();
-
-    # TODO ??? Verify
-    my @namespace = @{ $self->{globalNamespace} };
-    push @namespace, "statsd";
-    my $namespace = join(".", @namespace);
-
-    $stat_string .= sprintf("%s.graphiteStats.last_exception %d %d\n",
-        $namespace, $last_exception, $ts);
-    $stat_string .= sprintf("%s.graphiteStats.last_flush %d %d\n",
-        $namespace, $last_flush, $ts);
-
+    my $stat_string = $self->stats_to_string($stat_list);
     $graphite->send($stat_string);
     $graphite->close();
 
@@ -134,97 +257,64 @@ sub post_stats {
 
 }
 
-sub flush {
-  my ($self, $timestamp, $metrics) = @_;
+sub stat_float {
+  my ($stat, $val, $ts) = @_;
+  return {
+    stat  => $stat,
+    value => $val,
+    time  => $ts,
+    fmt   => fmt_FLOAT,
+  };
+}
 
-  my $ts_suffix = " $timestamp\n";
-  my $startTime = [Time::HiRes::gettimeofday];
-  my $statString = "";
-  my $numStats = 0;
-  my $timer_data_key;
+sub stat_int {
+  my ($stat, $val, $ts) = @_;
+  return {
+    stat  => $stat,
+    value => $val,
+    time  => $ts,
+    fmt   => fmt_INT,
+  };
+}
 
-  my $counters = $metrics->{counters};
-  my $gauges = $metrics->{gauges};
-  my $timers = $metrics->{timers};
-  my $sets = $metrics->{sets};
-  my $counter_rates = $metrics->{counter_rates};
-  my $timer_data = $metrics->{timer_data};
-  my $statsd_metrics = $metrics->{statsd_metrics};
+sub stat_str {
+  my ($stat, $val, $ts) = @_;
+  return {
+    stat  => $stat,
+    value => $val,
+    time  => $ts,
+    fmt   => fmt_STR,
+  };
+}
 
-  for my $key (keys %{ $counters }) {
+sub stat_time {
+  my ($stat, $val, $ts) = @_;
+  return {
+    stat  => $stat,
+    value => $val,
+    time  => $ts,
+    fmt   => fmt_TIME,
+  };
+}
 
-    my @namespace = (@{ $self->{counterNamespace} }, $key);
-    my $namespace = join(".", @namespace);
-
-    my $value = $counters->{$key};
-    my $valuePerSecond = $counter_rates->{$key}; # pre-calculated "per second" rate
-
-    if ($self->{legacyNamespace}) {
-      $statString .= "$namespace $valuePerSecond $ts_suffix";
-      $statString .= "stats_counts.$key $value $ts_suffix";
-    } else {
-      $statString .= "$namespace.rate $valuePerSecond $ts_suffix";
-      $statString .= "$namespace.count $value $ts_suffix";
-    }
-    $numStats++;
+sub stats_to_string {
+  my ($self, $stat_list) = @_;
+  my $stat_string = "";
+  for (@{ $stat_list }) {
+    my $attr = $_;
+    my $stat = $attr->{stat};
+    my $val  = $attr->{value};
+    my $ts   = $attr->{time};
+    my $fmt  = exists $attr->{fmt} ? $attr->{fmt} : '%d';
+    $stat_string .= sprintf("%s $fmt %d\n", $stat, $val, $ts);
   }
-
-  for my $key (keys %{ $timer_data }) {
-    if ($timer_data->{$key} && keys %{ $timer_data->{$key} } > 0) {
-      for my $timer_data_key (keys %{ $timer_data->{$key} }) {
-        my @namespace = (@{ $self->{timerNamespace} }, $key);
-        my $the_key = join(".", @namespace);
-        $statString .= "$the_key.$timer_data_key "
-          . $timer_data->{$key}->{$timer_data_key}
-          . $ts_suffix;
-      }
-      $numStats++;
-    }
-  }
-
-  for my $key (keys %{ $gauges }) {
-    my @namespace = (@{ $self->{gaugesNamespace} }, $key);
-    $statString .= join(".", @namespace) . " " . $gauges->{$key} . $ts_suffix;
-    $numStats++;
-  }
-
-  for my $key (keys %{ $sets }) {
-    my @namespace = (@{ $self->{setsNamespace} }, $key);
-    my $namespace = join(".", @namespace);
-    my $set_card = scalar keys %{ $sets->{$key} };
-    $statString .= "$namespace.count $set_card $ts_suffix";
-    $numStats++;
-  }
-
-  my @namespace = (@{ $self->{globalNamespace} }, "statsd");
-
-  # Convert Time::HiRes format (µs) to ms
-  my $calcTime = sprintf "%.6f", 1000 * Time::HiRes::tv_interval($startTime);
-
-  if ($self->{legacyNamespace}) {
-    $statString .= "statsd.numStats $numStats $ts_suffix";
-    $statString .= "stats.statsd.graphiteStats.calculationtime $calcTime $ts_suffix";
-    for my $key (keys %{ $statsd_metrics }) {
-      $statString .= "stats.statsd.$key $statsd_metrics->{$key} $ts_suffix";
-    }
-  }
-  else {
-    my $namespace = join(".", @namespace);
-    $statString .= "$namespace.numStats $numStats $ts_suffix";
-    $statString .= "$namespace.graphiteStats.calculationtime $calcTime $ts_suffix";
-    for my $key (keys %{ $statsd_metrics }) {
-      my $value = $statsd_metrics->{$key};
-      $statString .= "$namespace.$key $value $ts_suffix";
-    }
-  }
-  $self->post_stats($statString);
+  return $stat_string;
 }
 
 sub status {
   my ($self) = @_;
-  my $stats = $self->{graphiteStats};
   return {
-    last_flush => $self->since($self->{lastFlush}),
+    last_flush     => $self->since($self->{lastFlush}),
     last_exception => $self->since($self->{lastException}),
   };
 }
